using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Text;
using System.Windows.Forms;
using SpiderMind.Settings;

namespace SpiderMind
{
  public partial class frmMain
  {
    private double LastFPS = 0;
    private double FPS = 0;
    private uint LastFramesDrawn = 0;
    private uint FramesDrawn = 0;
    private float TickDelta = 0.25f;
    private double LastTick = (DateTime.Now.Ticks / (double)10000000);

    private int FormWidth;
    private int FormHeight;

    private bool IsDrawing = false;

    private float RotateAngle = 0;

    private bool MouseFocus;
    private bool MouseIsDown = false;
    private int MouseX;
    private int MouseY;

    private Bitmap WorkspaceBackground = new Bitmap(1,1);

    private enum NodeType
    {
      Active    = 0,
      Local     = 1,
      Default   = 2,
      Distant   = 3,
      Highlight = 4
    }

    private enum LinkType
    {
      Active    = 0,
      Local     = 1,
      Default   = 2,
      Distant   = 3,
      Highlight = 4
    }

    private void workspace_paint_throbber(Graphics myGraphics)
    {
      SmoothingMode mySmoothingMode = myGraphics.SmoothingMode;
      Pen myPen = new Pen(Throbber.Default.Color, Throbber.Default.Width);
      GraphicsPath myPath;

      if (Throbber.Default.DrawingMode != SmoothingMode.Invalid)
        myGraphics.SmoothingMode = Throbber.Default.DrawingMode;
      
      switch (Throbber.Default.Type)
      {
        case 0:
          RectangleF myRect = new RectangleF(
                                    -(Throbber.Default.Size / 2),
                                    -(Throbber.Default.Size / 2),
                                    Throbber.Default.Size,
                                    Throbber.Default.Size
                                  );
          myPath = new GraphicsPath(FillMode.Alternate);

          myPath.AddRectangle(myRect);
          myGraphics.TranslateTransform(this.Width / 2, this.Height / 2);
          myGraphics.RotateTransform(RotateAngle);
          myGraphics.DrawPath(myPen, myPath);
          myGraphics.RotateTransform(45);
          myGraphics.DrawPath(myPen, myPath);
          myGraphics.ResetTransform();

          myPath.Dispose();
          break;
        case 1:
          myPath = new GraphicsPath(FillMode.Alternate);

          float myMod = (float)(Math.Abs(RotateAngle - 180) / 180 * 0.5);

          myPath.AddLine(
            (float)(Throbber.Default.Size * 0.6 * myMod),
            (float)(Throbber.Default.Size * 0.6 * myMod),
            0,
            (float)-(Throbber.Default.Size * 0.6)
          );
          myPath.AddArc(
            (float)-(Throbber.Default.Size * 0.6),
            (float)-(Throbber.Default.Size * 0.6),
            (float)(Throbber.Default.Size * 1.2),
            (float)(Throbber.Default.Size * 1.2),
            270,
            110
          );
          myPath.AddLine(
            (float)(Throbber.Default.Size * 0.6 * myMod),
            (float)(Throbber.Default.Size * 0.6 * myMod),
            (float)(Throbber.Default.Size * 0.6 * myMod),
            (float)(Throbber.Default.Size * 0.6 * myMod)
          );

          myGraphics.TranslateTransform(this.Width / 2, this.Height / 2);
          myGraphics.RotateTransform(RotateAngle);
          myGraphics.DrawPath(myPen, myPath);
          myGraphics.RotateTransform(120);
          myGraphics.DrawPath(myPen, myPath);
          myGraphics.RotateTransform(120);
          myGraphics.DrawPath(myPen, myPath);
          myGraphics.ResetTransform();

          myPath.Dispose();
          break;
        default:
          break;
      }
      myGraphics.SmoothingMode = mySmoothingMode;

      myPen.Dispose();
    }

    private void workspace_cache_background()
    {
      int myMid = this.Height / 2;

      WorkspaceBackground.Dispose();
      WorkspaceBackground = new Bitmap(this.ClientRectangle.Width, this.ClientRectangle.Height);
      Graphics myGraphics = Graphics.FromImage(WorkspaceBackground);
      SmoothingMode mySmoothingMode = myGraphics.SmoothingMode;
      Rectangle myTopRect = new Rectangle(0, 0, this.Width, myMid);
      Rectangle myBotRect = new Rectangle(0, myMid - 1, this.Width, myMid + 1);
      LinearGradientBrush myTopGradient = new LinearGradientBrush(myTopRect, Workspace.Default.Color1, Workspace.Default.Color2, 270f);
      LinearGradientBrush myBotGradient = new LinearGradientBrush(myBotRect, Workspace.Default.Color1, Workspace.Default.Color2, 90f);

      myGraphics.SmoothingMode = SmoothingMode.Default;

      myGraphics.FillRectangle(myTopGradient, 0, 0, this.Width, this.Height / 2);
      myGraphics.FillRectangle(myBotGradient, 0, this.Height / 2, this.Width, this.Height);

      myGraphics.SmoothingMode = mySmoothingMode;

      myTopGradient.Dispose();
      myBotGradient.Dispose();
      myGraphics.Dispose();
    }

    private void workspace_paint_background(Graphics myGraphics)
    {
      if (WorkspaceBackground.Width != this.ClientRectangle.Width || WorkspaceBackground.Height != this.ClientRectangle.Height)
        workspace_cache_background();
      this.BackgroundImage = WorkspaceBackground;
      //myGraphics.DrawImage(WorkspaceBackground, 0, 0);
    }

    private void workspace_paint_node(Graphics myGraphics, NodeType myType, bool myDrawConnections, string Text, int X, int Y)
    {
      SizeF mySize;
      Pen myPen;
      Brush myBrush;
      Font myFont;
      Brush myFontBrush;
      RectangleF myRectF;
      StringFormat myTextFormat = new StringFormat();
      myTextFormat.Alignment = StringAlignment.Center;
      myTextFormat.LineAlignment = StringAlignment.Center;
      switch (myType)
      {
        case NodeType.Active:
          myPen = new Pen(Node.Default.ActiveBorderColor);
          myBrush = new SolidBrush(Node.Default.ActiveBackgroundColor);
          myFont = new Font(Node.Default.ActiveFont, Node.Default.ActiveFont.Style);
          myFontBrush = new SolidBrush(Node.Default.ActiveColor);
          break;
        case NodeType.Local:
          myPen = new Pen(Node.Default.DefaultBorderColor);
          myBrush = new SolidBrush(Node.Default.DefaultBackgroundColor);
          myFont = new Font(Node.Default.DefaultFont, Node.Default.DefaultFont.Style);
          myFontBrush = new SolidBrush(Node.Default.ActiveColor);
          break;
        case NodeType.Distant:
          myPen = new Pen(Node.Default.DistantBorderColor);
          myBrush = new SolidBrush(Node.Default.DistantBackgroundColor);
          myFont = new Font(Node.Default.DistantFont, Node.Default.DistantFont.Style);
          myFontBrush = new SolidBrush(Node.Default.DistantColor);
          break;
        case NodeType.Highlight:
          myPen = new Pen(Node.Default.HighlightBorderColor);
          myBrush = new SolidBrush(Node.Default.HighlightBackgroundColor);
          myFont = new Font(Node.Default.HighlightFont, Node.Default.HighlightFont.Style);
          myFontBrush = new SolidBrush(Node.Default.HighlightColor);
          break;
        default:
          myPen = new Pen(Node.Default.DefaultBorderColor);
          myBrush = new SolidBrush(Node.Default.DefaultBackgroundColor);
          myFont = new Font(Node.Default.DefaultFont, Node.Default.DefaultFont.Style);
          myFontBrush = new SolidBrush(Node.Default.ActiveColor);
          break;
      }

      mySize = myGraphics.MeasureString(Text, myFont);
      myRectF = new RectangleF(
        X - (mySize.Width / 2) - Node.Default.NodePadding,
        Y - (mySize.Height / 2) - Node.Default.NodePadding,
        mySize.Width + (Node.Default.NodePadding * 2),
        mySize.Height + (Node.Default.NodePadding * 2)
      );

      myGraphics.FillRectangle(myBrush, myRectF);
      myGraphics.DrawString(Text, myFont, myFontBrush, X, Y, myTextFormat);
      myGraphics.DrawRectangle(myPen, myRectF.Left, myRectF.Top, myRectF.Width, myRectF.Height);

      myTextFormat.Dispose();
      myPen.Dispose();
      myBrush.Dispose();
      myFontBrush.Dispose();
      myFont.Dispose();
    }

    private void workspace_paint_link(Graphics myGraphics, LinkType myType, float xFrom, float yFrom, float xTo, float yTo)
    {
      float[] hitX = { xFrom - 5, xFrom + 5, xTo + 5, xTo - 5 };
      float[] hitY = { yFrom, yFrom, yTo, yTo };

      Pen myPen = new Pen(Node.Default.ActiveBorderColor);
      Pen myPenHilite = new Pen(Node.Default.LinkHighlightColor, Node.Default.LinkHighlightWidth);
      Pen myPenActive = new Pen(Node.Default.LinkActiveColor, Node.Default.LinkActiveWidth);

      if (hittest(hitX, hitY, MouseX, MouseY))
        myGraphics.DrawLine(myPenHilite, xFrom, yFrom, xTo, yTo);
      else
        myGraphics.DrawLine(myPen, xFrom, yFrom, xTo, yTo);

      myPen.Dispose();
      myPenHilite.Dispose();
    }

    private void paint_Button(Graphics formGraphics, int X, int Y, int Size)
    {
      Pen myPen = new Pen(Color.Blue);
      Brush myBrush;
      Rectangle myButton = new Rectangle(X, Y, Size, Size);

      // TODO: Configurable colors, etc...
      if (MouseFocus && (MouseX >= myButton.Left && MouseX <= myButton.Right) && (MouseY >= myButton.Top && MouseY <= myButton.Bottom))
        myBrush = new SolidBrush(Color.Black);
      else
        myBrush = new SolidBrush(Color.White);

      formGraphics.FillRectangle(myBrush, myButton);
      formGraphics.DrawRectangle(myPen, myButton);
    }

    private bool hittest(float[] xp, float[] yp, float x, float y)
    {
      int i, j, npol = xp.Length;
      bool c = false;
      if (xp.Length == yp.Length) // Just in case...
        for (i = 0, j = npol - 1; i < npol; j = i++)
        {
          if ((((yp[i] <= y) && (y < yp[j])) ||
              ((yp[j] <= y) && (y < yp[i]))) &&
              (x < (xp[j] - xp[i]) * (y - yp[i]) / (yp[j] - yp[i]) + xp[i]))
            c = !c;
        }
      return c;
    }
  }
}